/*************************************************************************
 * $Workfile: thai.h$
 *************************************************************************
 *
 * Copyright(C) Razorcat Development GmbH. All Rights Reserved.
 *
 * The copyright to the computer program(s) herein is the property of
 * Razorcat Development GmbH, Germany. The program(s) may be used and/or
 * copied only with the written permission of Razorcat Development
 * GmbH or in accordance with the terms and conditions stipulated in
 * the agreement/contract under which the program(s) have been supplied.
 * This copyright notice must not be removed.
 *************************************************************************
 *
 * Description:   This file provides the Tessy hardware adapter
 *                interface (THAI) used for tests with stimulation
 *                and measurement hardware equipment.
 *
 * Dependencies:  Tessy hardware adapter interface (THAI) specification
 *
 * $Author: Thorsten Miesel$
 * $Revision: 5$
 *
 *************************************************************************/

/*!\file
 *    This file provides the interface of the Tessy hardware abstraction
 *    layer (THAI) used for tests with stimulation and measurement hardware
 *    equipment.
 * \brief   Tessy hardware abstraction layer (THAI).
 */





#ifdef __cplusplus
   extern "C" {
#endif

#ifndef TESSY
#ifdef THAI_EXPORTS
#define THAI_API __declspec(dllexport)
#else
#define THAI_API __declspec(dllimport)
#endif
#else
#define THAI_API
#endif

/****************************************************************************
 *
 * Definitions common to all functions of the THAI-API
 *
 ****************************************************************************/

// Return values of the THAI functions
#define THAI_OK			0
#define THAI_ERR			1
// Every return code other than THAI_OK will be treated as an error. A
// detailed error message will be provided by THAI_get_last_error().

// Data types
#ifndef TESSY
typedef unsigned char uint8;			// 8 bit unsigned
typedef unsigned short int uint16;	// 16 bit unsigned
typedef unsigned long int uint32;	// 32 bit unsigned
typedef signed char int8;				// 8 bit signed
typedef signed short int int16;		// 16 bit signed
typedef signed long int int32;		// 32 bit signed
#endif

// The data type returned by the THAI-API functions
typedef int8 THAI_ret;




/************************************************************************/
/*!         Specifies the log level within three possible levels from
 *          detailed to none.
 * \brief   Log level for THAI activities
*/
/************************************************************************/

// The log level defines how much of the activity of the THAI-API
// functions shall be logged. When the filename argument is NULL, the
// logging will be redirected to stdio.
//
// Log every activity
#define THAI_LOG_LEVEL_DETAIL		2
//
// Log main activity
#define THAI_LOG_LEVEL_SUMMARY	1
//
// Log nothing.
#define THAI_LOG_LEVEL_NOTHING	0
//

THAI_API THAI_ret  THAI_specify_logging(uint8 log_level, const char *filename);
typedef THAI_ret(*P_THAI_specify_logging)(uint8, const char*);


/************************************************************************/
/*!         Establishes a connection to the THAI device. The connection
 *          parameters are specified within a configuration file passed
 *          as parameter.
 * \brief   Connecting to THAI device
*/
/************************************************************************/

THAI_API THAI_ret  THAI_connect(const char *config_file_name);
typedef THAI_ret(*P_THAI_connect)(const char*);


/************************************************************************/
/*!         Initializes a (new) sequence of setting signal values,
 *          flushing to the hardware, aquiring measurements from the
 *          hardware and reading signal values.
 * \brief   Resetting stimulation/measurement activities
*/
/************************************************************************/

THAI_API THAI_ret  THAI_reset_signals(void);
typedef THAI_ret(*P_THAI_reset_signals)(void);


/************************************************************************/
/*!         Sets the value of the given signal.
 * \brief   Setting a signal value
*/
/************************************************************************/

THAI_API THAI_ret  THAI_set_signal(const char *name, const char *value);
typedef THAI_ret (*P_THAI_set_signal)(const char*, const char*);


/************************************************************************/
/*!         Flushes the stimulation signal values to the hardware.
 * \brief   Flushing signals to hardware
*/
/************************************************************************/

THAI_API THAI_ret  THAI_flush_signals(void);
typedef THAI_ret (*P_THAI_flush_signals)(void);


/************************************************************************/
/*!         Acquires signal measurement data from the hardware.
 * \brief   Aquiring signal measurement data
*/
/************************************************************************/

THAI_API THAI_ret  THAI_get_signal_measurements(void);
typedef THAI_ret (*P_THAI_get_signal_measurements)(void);


/************************************************************************/
/*!         Returns the measured value of the given signal.
 * \brief   Reading a signal value
*/
/************************************************************************/

THAI_API THAI_ret  THAI_get_signal(const char *name, char **value);
typedef THAI_ret (*P_THAI_get_signal)(const char*, char**);


/************************************************************************/
/*!         Closes a connection to the THAI device.
 * \brief   Disconnecting from THAI device
*/
/************************************************************************/

THAI_API THAI_ret  THAI_disconnect(void);
typedef THAI_ret(*P_THAI_disconnect)(void);


/************************************************************************/
/*!         Returns a text explaining the last error that has occurred.
 *          If no error occurred, the function will return an empty
 *          string.
 * \brief   Returning the last error message text
*/
/************************************************************************/

THAI_API const char *THAI_get_last_error(void);
typedef const char *(*P_THAI_get_last_error)(void);


/************************************************************************/
/*!         Returns a text explaining the last error that has occurred.
 *          If no error occurred, the function will return an empty
 *          string.
 * \brief   Returning the last error message text
*/
/************************************************************************/

THAI_API THAI_ret  THAI_get_time(uint8 timer_number,
											uint16 *time_value,
											uint16 *time_resolution);
typedef THAI_ret (*P_THAI_get_time)(uint8, uint16*, uint16*);


#ifdef __cplusplus
}
#endif

/*************************************************************************
 * File change history
 *************************************************************************

$Log: 
 5    V4.1      2018  Thorsten Miesel Adapted to TESSY v4.1
 4    V3.0      1.2.1.0     22.04.2010 17:12:02  Thorsten Miesel Merge from
      thai.h, Revision 1.1.1.0
 3    V2.9 Bugfix1.2         24.03.2010 18:14:07  Thorsten Miesel work with
      function pointers for THAI
 2    V2.9 Bugfix1.1         21.01.2010 19:28:22  Thorsten Miesel changed return
      type of THAI_get_last_error
 1    V2.9 Bugfix1.0         21.01.2010 13:16:11  Michael Wittner 
$
*/
