/**
 * THAI dummy DLL to simulate signals without real hardware.
 *
 * Copyright Razorcat Development GmbH, 2018
 */

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "thai.h"

/* variable for signals
 * input signal values are added to output and output is given back
 * as first expected signal, and incremented for each following expected signal
 */
static unsigned char output = 0;

char *logFile   = NULL;
FILE *logHandle = NULL;
char logLevel   = 0;

/* log message to tessy console.
   (stdout is not available because the dll is loaded via LoadLibrary) */
static void TESSYout(const char *title, const char *format, ...) {
   DWORD nBytes;
   char message[1024*8];
   va_list   ap;

   va_start (ap, format);
   vsprintf(message, format, ap);
	va_end (ap);
	
	if (logLevel > 0) {
		WriteFile(GetStdHandle(STD_OUTPUT_HANDLE), message, strlen(message), &nBytes, 0);
	}

   if (logLevel == 2) {
      MessageBox(0, message, title, MB_OK);
   }
}

THAI_ret  THAI_specify_logging(uint8 log_level, const char *filename)
{
	logLevel = log_level;
  	TESSYout(__func__, "THAI: log_level is %d, log file is '%s'.\n", log_level, filename);
   if (filename && strlen(filename)) {
      logFile = strdup(filename);
   }
   else {
		char *testArea = getenv("TESSY_TESTAREA");
		if (testArea) {
			int length = strlen(testArea) + sizeof("\\thai.log") + 1;
			logFile = (char *)malloc(length);
			if (logFile) {
				sprintf(logFile, "%s\\thai.log", testArea);
			}
		}
   }
	logHandle = fopen(logFile, "w");
   if (!logHandle) {
      TESSYout(__func__, "THAI: Error opening THAI log file %s\n", logFile);
   }
	return THAI_OK;
}


THAI_ret  THAI_connect(const char *config_file_name)
{
  	TESSYout(__func__, "THAI: Connect to device specified in THAI Configuration File '%s'.\n", config_file_name);
	if (logHandle) {
   	fprintf(logHandle, "Connect to device specified in THAI Configuration File %s.\n", config_file_name);
   }
	return THAI_OK;
}



THAI_ret  THAI_reset_signals(void)
{
	TESSYout(__func__, "THAI: resetting signals.\n");
	if (logHandle) {
   	fprintf(logHandle, "Resetting signals.\n");
   }
	output = 0;
	return THAI_OK;
}



THAI_ret  THAI_set_signal(const char *name, const char *value)
{
   output += atoi(value);
	TESSYout(__func__, "THAI: set signal '%s' to '%s' (%d)\n", name, value, output);
	if (logHandle) {
	   fprintf(logHandle, "set signal %s to %s (checkvalue = %d).\n", name, value, output);
	}
	return THAI_OK;
}



THAI_ret  THAI_flush_signals(void)
{
	TESSYout(__func__, "THAI: flush the stimulation signal values to the hardware.\n");
	if (logHandle) {
	   fprintf(logHandle, "Flush the stimulation signal values to the hardware.\n");
	}

	return THAI_OK;
}


THAI_ret  THAI_get_signal_measurements(void)
{
   TESSYout(__func__, "THAI: acquire signal measurement data from the hardware.\n");
   if (logHandle) {
	   fprintf(logHandle, "Acquire signal measurement data from the hardware.\n");
	}

	return THAI_OK;
}




THAI_ret  THAI_get_signal(const char *name, char **value)
{
   output += 1;
	sprintf(*value, "%d", output);

	TESSYout(__func__, "THAI: get signal '%s' -> '%s' (%d)\n", name, *value, output);
   if (logHandle) {
      fprintf(logHandle, "get signal %s (%s) (checkvalue = %d)\n", name, *value, output);
   }
	return THAI_OK;
}



THAI_ret  THAI_disconnect(void)
{
   TESSYout(__func__, "THAI: disconnect from THAI device.\n");
   if (logHandle) {
   	fprintf(logHandle, "disconnect from THAI device.\n");
      fflush(logHandle);
		fclose(logHandle);
   }
	if (logFile) {
		free(logFile);
		logFile = NULL;
	}
	return THAI_OK;
}


const char *THAI_get_last_error(void)
{
	static char error_message[1024];

	sprintf(error_message, "THAI No Error");
	return error_message;
}



THAI_ret  THAI_get_time(uint8 timer_number,
											uint16 *time_value,
											uint16 *time_resolution)
{
	*time_value += 1;
	TESSYout(__func__, "THAI: timer number: %d\n", timer_number);
	return THAI_ERR;
}
